@@ -198,14 +198,27 @@ class TriangleMesh:
 
         fcolors = self.face_data.colors
         if fcolors.shape[-1] == 3 and len(fcolors.shape) == 2:
+            # vcglib face colors have an inverted alpha channel (so 1 = transparent, 0 = opaque)
             fcolors = np.concatenate([np.ascontiguousarray(fcolors),
-                                      np.ones([fcolors.shape[0], 1], dtype=self.face_data.colors.dtype)], axis=-1)
+                                      np.zeros([fcolors.shape[0], 1], dtype=self.face_data.colors.dtype)], axis=-1)
+        elif fcolors.shape[-1] == 4 and len(fcolors.shape) == 2:
+            fcolors[:, -1] = 1.0 - fcolors[:, -1]
 
         wcolors = self.face_data.wedge_colors
         if wcolors.shape[-1] == 3 and len(wcolors.shape) == 3:
             wcolors = np.concatenate([np.ascontiguousarray(wcolors),
                                       np.ones([wcolors.shape[0], wcolors.shape[1], 1], dtype=wcolors.dtype)], axis=-1)
 
+        if fcolors.shape[0] > 0:
+            if fcolors.max() > 1.0 or fcolors.min() < 0.0:
+                raise ValueError("Invalid values for face colors, must be between 0 and 1 (inclusive)")
+        if vcolors.shape[0] > 0:
+            if vcolors.max() > 1.0 or vcolors.min() < 0.0:
+                raise ValueError("Invalid values for vertex colors, must be between 0 and 1 (inclusive)")
+        if wcolors.shape[0] > 0:
+            if wcolors.max() > 1.0 or wcolors.min() < 0.0:
+                raise ValueError("Invalid values for wedge colors, must be between 0 and 1 (inclusive)")
+
         save_mesh_internal(filename,
                            np.ascontiguousarray(self.vertex_data.positions.astype(dtype)),
                            np.ascontiguousarray(self.vertex_data.normals.astype(dtype)),
@@ -396,5 +409,5 @@ def load_mesh_vfn(filename, dtype=float):
 
 def load_mesh_vfnc(filename, dtype=float):
     ret = load_triangle_mesh(filename, dtype=dtype)
-    return ret.v, ret.f, ret.vn, ret.c
+    return ret.v, ret.f, ret.vn, ret.vc
 
